@@ -27,6 +27,7 @@
 // TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 
 using System;
+using System.Collections.Generic;
 using System.ComponentModel;
 using System.Data;
 using System.Data.Common;
@@ -56,7 +57,7 @@ namespace Npgsql
         // Logging related values
         private static readonly String CLASSNAME = MethodBase.GetCurrentMethod().DeclaringType.Name;
         private static readonly ResourceManager resman = new ResourceManager(MethodBase.GetCurrentMethod().DeclaringType);
-        private static readonly Regex parameterReplace = new Regex(@"([:@][\w\.]*)", RegexOptions.Singleline);
+        private static readonly Regex parameterReplace = new Regex(@"([:@][\w\.]*)", RegexOptions.Singleline|RegexOptions.Compiled);
         private static readonly Regex POSTGRES_TEXT_ARRAY = new Regex(@"^array\[+'", RegexOptions.Compiled | RegexOptions.CultureInvariant);
 
         private NpgsqlConnection connection;
@@ -81,14 +82,12 @@ namespace Npgsql
         
         private Boolean addProcedureParenthesis = false; // Do not add procedure parenthesis by default.
 
-        private Boolean functionReturnsRecord = false; // Functions don't return record by default.
-
-        private Boolean functionReturnsRefcursor = false; // Functions don't return refcursor by default.
-
-
+        private Boolean functionNeedsColumnListDefinition = false; // Functions don't return record by default.
 
         private Boolean commandTimeoutSet = false;
 
+        private UpdateRowSource updateRowSource = UpdateRowSource.Both;
+
 
         // Constructors
 
@@ -157,8 +156,10 @@ namespace Npgsql
             this.m_Connector = connector;
             type = CommandType.Text;
 
+            // Removed this setting. It was causing too much problem.
+            // Do internal commands really need different timeout setting?
             // Internal commands aren't affected by command timeout value provided by user.
-            timeout = 20;
+            // timeout = 20;
         }
 
         // Public properties.
@@ -205,8 +206,6 @@ namespace Npgsql
                 timeout = value;
                 NpgsqlEventLog.LogPropertySet(LogLevel.Debug, CLASSNAME, "CommandTimeout", value);
 
-
-
                 commandTimeoutSet = true;
 
             }
@@ -299,6 +298,8 @@ namespace Npgsql
             }
         }
 
+        internal Type[] ExpectedTypes { get; set; }
+
         protected override DbParameterCollection DbParameterCollection
         {
             get { return Parameters; }
@@ -379,12 +380,26 @@ namespace Npgsql
             {
                 NpgsqlEventLog.LogPropertyGet(LogLevel.Debug, CLASSNAME, "UpdatedRowSource");
 
-                return UpdateRowSource.Both;
+                return updateRowSource;
+            }
+            set 
+            {
+                switch (value)
+                {
+                        // validate value (required based on base type contract)
+                    case UpdateRowSource.None:
+                    case UpdateRowSource.OutputParameters:
+                    case UpdateRowSource.FirstReturnedRecord:
+                    case UpdateRowSource.Both:
+                        updateRowSource = value;
+                        break;
+                    default:
+                        throw new ArgumentOutOfRangeException();
+                }
             }
-
-            set { }
         }
 
+
         /// <summary>
         /// Returns oid of inserted row. This is only updated when using executenonQuery and when command inserts just a single row. If table is created without oids, this will always be 0.
         /// </summary>
@@ -442,6 +457,10 @@ namespace Npgsql
             clone.CommandTimeout = CommandTimeout;
             clone.CommandType = CommandType;
             clone.DesignTimeVisible = DesignTimeVisible;
+            if (ExpectedTypes != null)
+            {
+                clone.ExpectedTypes = (Type[])ExpectedTypes.Clone();
+            }
             foreach (NpgsqlParameter parameter in Parameters)
             {
                 clone.Parameters.Add(parameter.Clone());
@@ -586,19 +605,34 @@ namespace Npgsql
 
                 using (m_Connector.BlockNotificationThread())
                 {
+                    ForwardsOnlyDataReader reader;
                     if (parse == null)
                     {
-                        return new ForwardsOnlyDataReader(m_Connector.QueryEnum(this), cb, this, m_Connector.BlockNotificationThread(), false);
+                        reader = new ForwardsOnlyDataReader(m_Connector.QueryEnum(this), cb, this,
+                                                            m_Connector.BlockNotificationThread(), false);
+                        if (type == CommandType.StoredProcedure
+                            && reader.FieldCount == 1
+                            && reader.GetDataTypeName(0) == "refcursor")
+                        {
+                            // When a function returns a sole column of refcursor, transparently
+                            // FETCH ALL from every such cursor and return those results.
+                            StringBuilder sb = new StringBuilder();
+                            while (reader.Read())
+                            {
+                                sb.Append("fetch all from \"").Append(reader.GetString(0)).Append("\";");
+                            }
+                            sb.Append(";"); // Just in case the list of cursors is empty.
+
+                            reader = new NpgsqlCommand(sb.ToString(), Connection).GetReader(reader._behavior);
+                        }
                     }
-                    //return new ForwardsOnlyDataReader(m_Connector.QueryEnum(this), cb, this, m_Connector.BlockNotificationThread(), false);
                     else
                     {
                         BindParameters();
-                        return
-                            new ForwardsOnlyDataReader(m_Connector.ExecuteEnum(new NpgsqlExecute(bind.PortalName, 0)), cb, this,
-                                                       m_Connector.BlockNotificationThread(), true);
-                        //return new ForwardsOnlyDataReader(m_Connector.ExecuteEnum(new NpgsqlExecute(bind.PortalName, 0)), cb, this, m_Connector.BlockNotificationThread(), true);
+                        reader = new ForwardsOnlyDataReader(m_Connector.ExecuteEnum(new NpgsqlExecute(bind.PortalName, 0)), cb, this,
+                                                            m_Connector.BlockNotificationThread(), true);
                     }
+                    return reader;
                 }
             }
             catch (IOException ex)
@@ -647,9 +681,23 @@ namespace Npgsql
                 bind.ParameterFormatCodes = parameterFormatCodes;
             }
 
-            Connector.Bind(bind);
+            try
+            {
+                // In case of error when binding parameters, the ReadyForQuery isn't returned.
+                // According to docs: "[...] The response is either BindComplete or ErrorResponse."
 
-            Connector.Flush();
+                Connector.RequireReadyForQuery = false;
+
+                Connector.Bind(bind);
+
+                Connector.Flush();
+            }
+            catch
+            {
+                // Check catch{} of Preapre method for discussion about that.
+                Connector.Sync();
+                throw;
+            }
         }
 
         /// <summary>
@@ -766,7 +814,19 @@ namespace Npgsql
                     }
                     catch
                     {
-                        // See ExecuteCommand method for a discussion of this.
+
+                        // As per documentation:
+
+                        // "[...] When an error is detected while processing any extended-query message,
+
+                        // the backend issues ErrorResponse, then reads and discards messages until a
+
+                        // Sync is reached, then issues ReadyForQuery and returns to normal message processing.[...]"
+
+                        // So, send a sync command if we get any problems.
+
+
+
                         m_Connector.Sync();
 
                         throw;
@@ -828,7 +888,12 @@ namespace Npgsql
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetCommandText");
 
-            return string.IsNullOrEmpty(planName) ? GetClearCommandText() : GetPreparedCommandText();
+            StringBuilder ret = string.IsNullOrEmpty(planName) ? GetClearCommandText() : GetPreparedCommandText();
+            // In constructing the command text, we potentially called internal
+            // queries.  Reset command timeout and SQL sent.
+            m_Connector.Mediator.ResetResponses();
+            m_Connector.Mediator.CommandTimeout = CommandTimeout;
+            return ret;
 
         }
 
@@ -862,6 +927,47 @@ namespace Npgsql
             query.Append(array[endAt]);
         }
         
+        private void PassParam(StringBuilder query, NpgsqlParameter p)
+        {
+                string serialised = p.TypeInfo.ConvertToBackend(p.Value, false);
+
+                // Add parentheses wrapping parameter value before the type cast to avoid problems with Int16.MinValue, Int32.MinValue and Int64.MinValue
+                // See bug #1010543
+                // Check if this parenthesis can be collapsed with the previous one about the array support. This way, we could use
+                // only one pair of parentheses for the two purposes instead of two pairs.
+                query.Append('(');
+
+                if(Connector.UseConformantStrings)
+                    switch(serialised[0])
+                    {
+                    case '\''://type passed as string or string with type.
+                        //We could test to see if \ is used anywhere, but then we could be doing quite an expensive check (if the value is large) for little gain.
+                        query.Append("E").Append(serialised);
+                        break;
+                    case 'a':
+                        if(POSTGRES_TEXT_ARRAY.IsMatch(serialised))
+                            PassEscapedArray(query, serialised);
+                        else
+                            query.Append(serialised);
+                        break;
+                    default:
+                        query.Append(serialised);
+                        break;
+                    }
+                else
+                    query.Append(serialised);
+
+                query.Append(')');
+
+
+                if (p.UseCast)
+                {
+                    query.Append("::").Append(p.TypeInfo.CastName);
+                    if (p.TypeInfo.UseSize && (p.Size > 0))
+                        query.Append('(').Append(p.Size).Append(')');
+                }
+        }
+
         private StringBuilder GetClearCommandText()
         {
             if (NpgsqlEventLog.Level == LogLevel.Debug)
@@ -878,9 +984,7 @@ namespace Npgsql
                 case CommandType.StoredProcedure:
                     if (!functionChecksDone)
                     {
-                        functionReturnsRecord = Parameters.Count != 0 && !CheckFunctionHasOutParameters() && CheckFunctionReturn("record");
-
-                        functionReturnsRefcursor = CheckFunctionReturn("refcursor");
+                        functionNeedsColumnListDefinition = Parameters.Count != 0 && CheckFunctionNeedsColumnDefinitionList();
 
                         // Check if just procedure name was passed. If so, does not replace parameter names and just pass parameter values in order they were added in parameters collection. Also check if command text finishes in a ";" which would make Npgsql incorrectly append a "()" when executing this command text.
                         switch(result[result.Length - 1])
@@ -909,14 +1013,8 @@ namespace Npgsql
                 if (addProcedureParenthesis)
                     result.Append("()");
 
-                // If function returns ref cursor just process refcursor-result function call
-                // and return command which will be used to return data from refcursor.
-
-                if (functionReturnsRefcursor)
-                    return ProcessRefcursorFunctionReturn(result.ToString());
-
-                if (functionReturnsRecord)
-                    AddFunctionReturnsRecordSupport(result);
+                if (functionNeedsColumnListDefinition)
+                    AddFunctionColumnListSupport(result);
                
                 return result;
             }
@@ -932,65 +1030,36 @@ namespace Npgsql
             // If parenthesis don't need to be added, they were added by user with parameter names. Replace them.
             if (!addProcedureParenthesis)
             {
+                Dictionary<string, NpgsqlParameter> parameterIndex = new Dictionary<string, NpgsqlParameter>(parameters.Count, StringComparer.InvariantCultureIgnoreCase);
+                foreach (NpgsqlParameter parameter in parameters)
+                    parameterIndex[parameter.CleanName] = parameter;
+
                 StringBuilder sb = new StringBuilder();
                 foreach (String s in parameterReplace.Split(result.ToString()))
                     if (s.Length != 0)
                     {
-                        NpgsqlParameter p;
-                        if ((s[0] == ':' || s[0] == '@') && Parameters.TryGetValue(s, out p))
+                        NpgsqlParameter p = null;
+                        string parameterName = s;
+                        if ((parameterName[0] == ':') || (parameterName[0] == '@'))
+                        {
+                            parameterName = parameterName.Remove(0, 1);
+                            parameterIndex.TryGetValue(parameterName, out p);
+                        }
+
+                        if (p != null)
                         {
-                            // It's a parameter. Lets handle it.
                             switch(p.Direction)
                             {
                                 case ParameterDirection.Input: case ParameterDirection.InputOutput:
-                                    //Start the probably-redundant parenthesis. Queries should operate much as if they were in the a parameter or
+                                    //Wrap in probably-redundant parentheses. Queries should operate much as if they were in the a parameter or
                                     //variable in a postgres function. Generally this is the case without the parentheses (hence "probably redundant")
                                     //but there are exceptions to this rule. E.g. consider the postgres function:
                                     //
                                     //CREATE FUNCTION first_param(integer[])RETURNS int AS'select $1[1]'LANGUAGE 'sql' STABLE STRICT;
                                     //
                                     //The equivalent commandtext would be "select :param[1]", but this fails without the parentheses.
-                                    sb.Append('(');
-                                    
-                                    string serialised = p.TypeInfo.ConvertToBackend(p.Value, false);
 
-                                    // Add parentheses wrapping parameter value before the type cast to avoid problems with Int16.MinValue, Int32.MinValue and Int64.MinValue
-                                    // See bug #1010543
-                                    // Check if this parenthesis can be collapsed with the previous one about the array support. This way, we could use
-                                    // only one pair of parentheses for the two purposes instead of two pairs.
-                                    sb.Append('(');
-
-                                    if(Connector.UseConformantStrings)
-                                        switch(serialised[0])
-                                        {
-                                            case '\''://type passed as string or string with type.
-                                            //We could test to see if \ is used anywhere, but then we could be doing quite an expensive check (if the value is large) for little gain.
-                                                sb.Append("E").Append(serialised);
-                                                break;
-                                            case 'a':
-                                                if(POSTGRES_TEXT_ARRAY.IsMatch(serialised))
-                                                    PassEscapedArray(sb, serialised);
-                                                else
-                                                    sb.Append(serialised);
-                                                break;
-                                            default:
-                                                sb.Append(serialised);
-                                                break;
-                                        }
-                                    else
-                                        sb.Append(serialised);
-
-                                    sb.Append(')');
-
-                                    
-                                    if (p.UseCast)
-                                    {
-                                        sb.Append("::").Append(p.TypeInfo.CastName);
-                                        if (p.TypeInfo.UseSize && (p.Size > 0))
-                                            sb.Append('(').Append(p.Size).Append(')');
-                                    }
-
-                                    sb.Append(')');//Close probably-redundant parenthesis.
+                                    sb.Append('('); PassParam(sb, p); sb.Append(')');
                                     break;
                             }
                         }
@@ -1008,29 +1077,15 @@ namespace Npgsql
                 
                 for (Int32 i = 0; i < parameters.Count; i++)
                 {
-                    NpgsqlParameter Param = parameters[i];
-
-
-                    switch(Param.Direction)
+                    switch(parameters[i].Direction)
                     {
                         case ParameterDirection.Input: case ParameterDirection.InputOutput:
-                            // Add parentheses wrapping parameter value before the type cast to avoid problems with Int16.MinValue, Int32.MinValue and Int64.MinValue
-                            // See bug #1010543
-                            result.Append('(');
-                            result.Append(Param.TypeInfo.ConvertToBackend(Param.Value, false));
-                            result.Append(')');
-                            if (Param.UseCast)
-                            {
-                                result.Append("::").Append(Param.TypeInfo.CastName);
-                                if(Param.TypeInfo.UseSize && Param.Size > 0)
-                                    result.Append('(').Append(Param.Size).Append(')');
-                            }
+                            PassParam(result, parameters[i]);
                             result.Append(',');
                             break;
                     }
                 }
 
-
                 // Remove a trailing comma added from parameter handling above. If any.
                 // Maybe there are only output parameters. If so, there will be no comma.
                 if (result[result.Length - 1] == ',')
@@ -1038,65 +1093,53 @@ namespace Npgsql
                 result.Append(')');
             }
 
-            if (functionReturnsRecord)
+            if (functionNeedsColumnListDefinition)
             {
-                AddFunctionReturnsRecordSupport(result);
-            }
-
-            // If function returns ref cursor just process refcursor-result function call
-            // and return command which will be used to return data from refcursor.
-
-            if (functionReturnsRefcursor)
-            {
-                return ProcessRefcursorFunctionReturn(result.ToString());
+                AddFunctionColumnListSupport(result);
             }
 
             return result;
         }
 
-
-        private Boolean CheckFunctionHasOutParameters()
+        private Boolean CheckFunctionNeedsColumnDefinitionList()
         {
-            // Check if this function has output parameters.
-            // This is used to enable or not the colum definition list 
-            // when calling functions which return record.
-            // Functions which has out or inout parameters have return record
-            // but doesn't allow column definition list.
+            // If and only if a function returns "record" and has no OUT ("o" in proargmodes), INOUT ("b"), or TABLE
+            // ("t") return arguments to characterize the result columns, we must provide a column definition list.
             // See http://pgfoundry.org/forum/forum.php?thread_id=1075&forum_id=519
-            // for discussion about that.
+            // We would use our Output and InputOutput parameters to construct that column definition list.  If we have
+            // no such parameters, skip the check: we could only construct "AS ()", which yields a syntax error.
+
+            // Updated after 0.99.3 to support the optional existence of a name qualifying schema and allow for case insensitivity
+            // when the schema or procedure name do not contain a quote.
+            // The hard-coded schema name 'public' was replaced with code that uses schema as a qualifier, only if it is provided.
+
+            String returnRecordQuery;
+
+            StringBuilder parameterTypes = new StringBuilder("");
 
 
-            // inout parameters are only supported from 8.1+ versions.
-            if (Connection.PostgreSqlVersion < new Version(8, 1, 0))
+            // Process parameters
+
+            Boolean seenDef = false;
+            foreach (NpgsqlParameter p in Parameters)
+            {
+                if ((p.Direction == ParameterDirection.Input) || (p.Direction == ParameterDirection.InputOutput))
+                {
+                    parameterTypes.Append(Connection.Connector.OidToNameMapping[p.TypeInfo.Name].OID + " ");
+                }
+
+                if ((p.Direction == ParameterDirection.Output) || (p.Direction == ParameterDirection.InputOutput))
+                {
+                    seenDef = true;
+                }
+            }
+            
+            if (!seenDef)
             {
                 return false;
             }
 
 
-            //String outParameterExistanceQuery =
-            //    "select count(*) > 0 from pg_proc where proname=:proname and ('o' = any (proargmodes) OR 'b' = any(proargmodes))";
-
-
-            // Updated after 0.99.3 to support the optional existence of a name qualifying schema and allow for case insensitivity
-            // when the schema or procedure name do not contain a quote.
-            // The hard-coded schema name 'public' was replaced with code that uses schema as a qualifier, only if it is provided.
-
-            String returnRecordQuery;
-
-            StringBuilder parameterTypes = new StringBuilder("");
-
-
-            // Process parameters
-
-            foreach (NpgsqlParameter p in Parameters)
-            {
-                if ((p.Direction == ParameterDirection.Input) || (p.Direction == ParameterDirection.InputOutput))
-                {
-                    parameterTypes.Append(Connection.Connector.OidToNameMapping[p.TypeInfo.Name].OID + " ");
-                }
-            }
-
-
             // Process schema name.
 
             String schemaName = String.Empty;
@@ -1105,10 +1148,14 @@ namespace Npgsql
 
             String[] fullName = CommandText.Split('.');
 
+            String predicate = "prorettype = ( select oid from pg_type where typname = 'record' ) "
+                + "and proargtypes=:proargtypes and proname=:proname "
+                // proargmodes && array['o','b','t']::"char"[] performs just as well, but it requires PostgreSQL 8.2.
+                + "and ('o' = any (proargmodes) OR 'b' = any (proargmodes) OR 't' = any (proargmodes)) is not true";
             if (fullName.Length == 2)
             {
                 returnRecordQuery =
-                    "select count(*) > 0 from pg_proc p left join pg_namespace n on p.pronamespace = n.oid where prorettype = ( select oid from pg_type where typname = 'record' ) and proargtypes=:proargtypes and proname=:proname and n.nspname=:nspname and ('o' = any (proargmodes) OR 'b' = any(proargmodes))";
+                "select count(*) > 0 from pg_proc p left join pg_namespace n on p.pronamespace = n.oid where " + predicate + " and n.nspname=:nspname";
 
                 schemaName = (fullName[0].IndexOf("\"") != -1) ? fullName[0] : fullName[0].ToLower();
                 procedureName = (fullName[1].IndexOf("\"") != -1) ? fullName[1] : fullName[1].ToLower();
@@ -1118,83 +1165,7 @@ namespace Npgsql
                 // Instead of defaulting don't use the nspname, as an alternative, query pg_proc and pg_namespace to try and determine the nspname.
                 //schemaName = "public"; // This was removed after build 0.99.3 because the assumption that a function is in public is often incorrect.
                 returnRecordQuery =
-                    "select count(*) > 0 from pg_proc p where prorettype = ( select oid from pg_type where typname = 'record' ) and proargtypes=:proargtypes and proname=:proname and ('o' = any (proargmodes) OR 'b' = any(proargmodes))";
-
-                procedureName = (CommandText.IndexOf("\"") != -1) ? CommandText : CommandText.ToLower();
-            }
-
-
-            NpgsqlCommand c = new NpgsqlCommand(returnRecordQuery, Connection);
-
-            c.Parameters.Add(new NpgsqlParameter("proargtypes", NpgsqlDbType.Oidvector));
-            c.Parameters.Add(new NpgsqlParameter("proname", NpgsqlDbType.Name));
-
-            
-            c.Parameters[0].Value = parameterTypes.ToString();
-            c.Parameters[1].Value = procedureName;
-
-            if (schemaName != null && schemaName.Length > 0)
-            {
-                c.Parameters.Add(new NpgsqlParameter("nspname", NpgsqlDbType.Name));
-                c.Parameters[2].Value = schemaName;
-            }
-
-
-            Boolean ret = (Boolean)c.ExecuteScalar();
-
-            // reset any responses just before getting new ones
-            m_Connector.Mediator.ResetResponses();
-
-            // Set command timeout.
-            m_Connector.Mediator.CommandTimeout = CommandTimeout;
-
-            return ret;
-        }
-
-        private Boolean CheckFunctionReturn(String ReturnType)
-        {
-            // Updated after 0.99.3 to support the optional existence of a name qualifying schema and allow for case insensitivity
-            // when the schema or procedure name do not contain a quote.
-            // The hard-coded schema name 'public' was replaced with code that uses schema as a qualifier, only if it is provided.
-
-            String returnRecordQuery;
-
-            StringBuilder parameterTypes = new StringBuilder("");
-
-
-            // Process parameters
-
-            foreach (NpgsqlParameter p in Parameters)
-            {
-                if ((p.Direction == ParameterDirection.Input) || (p.Direction == ParameterDirection.InputOutput))
-                {
-                    parameterTypes.Append(Connection.Connector.OidToNameMapping[p.TypeInfo.Name].OID + " ");
-                }
-            }
-
-
-            // Process schema name.
-
-            String schemaName = String.Empty;
-            String procedureName = String.Empty;
-
-
-            String[] fullName = CommandText.Split('.');
-
-            if (fullName.Length == 2)
-            {
-                returnRecordQuery =
-                    "select count(*) > 0 from pg_proc p left join pg_namespace n on p.pronamespace = n.oid where prorettype = ( select oid from pg_type where typname = :typename ) and proargtypes=:proargtypes and proname=:proname and n.nspname=:nspname";
-
-                schemaName = (fullName[0].IndexOf("\"") != -1) ? fullName[0] : fullName[0].ToLower();
-                procedureName = (fullName[1].IndexOf("\"") != -1) ? fullName[1] : fullName[1].ToLower();
-            }
-            else
-            {
-                // Instead of defaulting don't use the nspname, as an alternative, query pg_proc and pg_namespace to try and determine the nspname.
-                //schemaName = "public"; // This was removed after build 0.99.3 because the assumption that a function is in public is often incorrect.
-                returnRecordQuery =
-                    "select count(*) > 0 from pg_proc p where prorettype = ( select oid from pg_type where typname = :typename ) and proargtypes=:proargtypes and proname=:proname";
+                    "select count(*) > 0 from pg_proc p where " + predicate;
 
                 procedureName = (CommandText.IndexOf("\"") != -1) ? CommandText : CommandText.ToLower();
             }
@@ -1204,21 +1175,18 @@ namespace Npgsql
 
             using (NpgsqlCommand c = new NpgsqlCommand(returnRecordQuery, Connection))
             {
-                c.Parameters.Add(new NpgsqlParameter("typename", NpgsqlDbType.Name));
                 c.Parameters.Add(new NpgsqlParameter("proargtypes", NpgsqlDbType.Oidvector));
                 c.Parameters.Add(new NpgsqlParameter("proname", NpgsqlDbType.Name));
 
-                c.Parameters[0].Value = ReturnType;
-                c.Parameters[1].Value = parameterTypes.ToString();
-                c.Parameters[2].Value = procedureName;
+                c.Parameters[0].Value = parameterTypes.ToString();
+                c.Parameters[1].Value = procedureName;
 
                 if (schemaName != null && schemaName.Length > 0)
                 {
                     c.Parameters.Add(new NpgsqlParameter("nspname", NpgsqlDbType.Name));
-                    c.Parameters[3].Value = schemaName;
+                    c.Parameters[2].Value = schemaName;
                 }
 
-
                 ret = (Boolean)c.ExecuteScalar();
             }
 
@@ -1231,8 +1199,7 @@ namespace Npgsql
             return ret;
         }
 
-
-        private void AddFunctionReturnsRecordSupport(StringBuilder sb)
+        private void AddFunctionColumnListSupport(StringBuilder sb)
         {
             sb.Append(" as (");
             foreach (NpgsqlParameter p in Parameters)
@@ -1245,38 +1212,6 @@ namespace Npgsql
             sb[sb.Length - 1] = ')';
         }
 
-        ///<summary>
-        /// This methods takes a string with a function call witch returns a refcursor or a set of
-        /// refcursor. It will return the names of the open cursors/portals which will hold
-        /// results. In turn, it returns the string which is needed to get the data of this cursors
-        /// in form of one resultset for each cursor open. This way, clients don't need to do anything
-        /// else besides calling function normally to get results in this way.
-        ///</summary>
-        private StringBuilder ProcessRefcursorFunctionReturn(String FunctionCall)
-        {
-            StringBuilder sb = new StringBuilder();
-            using (NpgsqlCommand c = new NpgsqlCommand(FunctionCall, Connection))
-            {
-                using (NpgsqlDataReader dr = c.GetReader(CommandBehavior.SequentialAccess | CommandBehavior.SingleResult))
-                {
-                    while (dr.Read())
-                    {
-                        sb.Append("fetch all from \"").Append(dr.GetString(0)).Append("\";");
-                    }
-                }
-            }
-
-            sb.Append(";"); // Just in case there is no response from refcursor function return.
-
-            // reset any responses just before getting new ones
-            m_Connector.Mediator.ResetResponses();
-
-            // Set command timeout.
-            m_Connector.Mediator.CommandTimeout = CommandTimeout;
-
-            return sb;
-        }
-
 
         private StringBuilder GetPreparedCommandText()
         {
@@ -1542,9 +1477,7 @@ namespace Npgsql
 
         private void SetCommandTimeout()
         {
-
             if (commandTimeoutSet)
-
                 return;
 
             if (Connection != null)
@@ -1569,4 +1502,4 @@ namespace Npgsql
             set { designTimeVisible = value; }
         }
     }
-}
+}
\ No newline at end of file
