@@ -149,6 +149,14 @@ namespace Npgsql
 
         private readonly Dictionary<string, NpgsqlParameterStatus> _serverParameters =
             new Dictionary<string, NpgsqlParameterStatus>(StringComparer.InvariantCultureIgnoreCase);
+        
+        // For IsValid test
+        private readonly RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();
+        
+        // Did we already called cancelRequest?
+        private Boolean _cancelRequestCalled = false;
+
+        
 
 #if WINDOWS && UNMANAGED
 
@@ -211,9 +219,9 @@ namespace Npgsql
             get { return settings.UserName; }
         }
 
-        internal String Password
+        internal byte[] Password
         {
-            get { return settings.Password; }
+            get { return settings.PasswordAsByteArray; }
         }
 
         internal Boolean SSL
@@ -294,7 +302,7 @@ namespace Npgsql
             return CurrentState.QueryEnum(this, queryCommand);
         }
 
-        internal void Authenticate(string password)
+        internal void Authenticate(byte[] password)
         {
             CurrentState.Authenticate(this, password);
         }
@@ -350,8 +358,17 @@ namespace Npgsql
             {
                 // Here we use a fake NpgsqlCommand, just to send the test query string.
                 
+                // Get random test value.
+                Byte[] testBytes = new Byte[2];
+                rng.GetNonZeroBytes(testBytes);
+                String testValue = String.Format("Npgsql{0}{1}", testBytes[0], testBytes[1]);
+                
                 //Query(new NpgsqlCommand("select 1 as ConnectionTest", this));
-                new NpgsqlCommand("select 1", this).ExecuteScalar();
+                String compareValue = (String) new NpgsqlCommand("select '" + testValue + "'", this).ExecuteScalar();
+                
+                if (compareValue != testValue)
+                    return false;
+
                 // Clear mediator.
                 Mediator.ResetResponses();
                 this.RequireReadyForQuery = true;
@@ -590,6 +607,11 @@ namespace Npgsql
             set { _transaction = value; }
         }
 
+        internal Boolean SupportsApplicationName
+        {
+            get { return ServerVersion >= new Version(9, 0, 0); }
+        }
+
         /// <summary>
         /// Report whether the current connection can support prepare functionality.
         /// </summary>
@@ -605,6 +627,12 @@ namespace Npgsql
             set { _supportsSavepoint = value; } 
           
         }
+        
+        public Boolean CancelRequestCalled  {
+            get { return _cancelRequestCalled; }
+        }
+
+        
 
         /// <summary>
         /// This method is required to set all the version dependent features flags.
@@ -728,6 +756,24 @@ namespace Npgsql
                 NpgsqlCommand commandSearchPath = new NpgsqlCommand("SET SEARCH_PATH=" + settings.SearchPath, this);
                 commandSearchPath.ExecuteBlind();
             }
+            
+            if (!string.IsNullOrEmpty(settings.ApplicationName))
+             {
+                 if (!SupportsApplicationName)
+                 {
+                     //TODO
+                     //throw new InvalidOperationException(resman.GetString("Exception_ApplicationNameNotSupported"));
+                     throw new InvalidOperationException("ApplicationName not supported.");
+                 }
+ 
+                 if (settings.ApplicationName.Contains(";"))
+                 {
+                     throw new InvalidOperationException();
+                 }
+ 
+                 NpgsqlCommand commandApplicationName = new NpgsqlCommand("SET APPLICATION_NAME='" + settings.ApplicationName + "'", this);
+                 commandApplicationName.ExecuteBlind();
+             }
 
 
 
@@ -797,16 +843,26 @@ namespace Npgsql
 
         internal void CancelRequest()
         {
-            NpgsqlConnector CancelConnector = new NpgsqlConnector(settings, false, false);
+            
+            NpgsqlConnector cancelConnector = new NpgsqlConnector(settings, false, false);
 
-            CancelConnector._backend_keydata = BackEndKeyData;
+            cancelConnector._backend_keydata = BackEndKeyData;
 
+            try
+            {
+                // Get a raw connection, possibly SSL...
+                cancelConnector.CurrentState.Open(cancelConnector);
 
-            // Get a raw connection, possibly SSL...
-            CancelConnector.CurrentState.Open(CancelConnector);
-
-            // Cancel current request.
-            CancelConnector.CurrentState.CancelRequest(CancelConnector);
+                // Cancel current request.
+                cancelConnector.CurrentState.CancelRequest(cancelConnector);
+            }
+            finally
+            {
+                cancelConnector.CurrentState.Close(cancelConnector);
+                
+                _cancelRequestCalled = true;
+            }
+            
         }
 
 
